﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Runtime.Serialization;

using Newtonsoft.Json;

using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.Common.Entities;
using Vintagestory.API.MathTools;

namespace Automap
{
	public delegate void EntityDesignatonAction(ICoreClientAPI clientAPI, EntitiesOfInterest poi, BlockPos posn, Entity entity);

	/// <summary>
	/// Point of Interest Rule Designator
	/// </summary>
	[JsonObject(MemberSerialization.OptIn)]
	public class EntityDesignator
	{
		[JsonProperty]
		public Color Color;

		[JsonIgnore]
		public EntityDesignatonAction SpecialAction;

		[JsonProperty]
		public string SpecialActionName;

		[JsonProperty]
		public AssetLocation Pattern;

		[JsonProperty]
		public EnumEntityState? StateCheck;//Needed?

		[JsonProperty]
		public bool Enabled { get; set; }

		private EntityDesignator()
		{
			//throw new NotSupportedException();
		}

		public EntityDesignator(AssetLocation pattern, Color color, EnumEntityState? state)
		{
			Pattern = pattern;
			Color = color;
			StateCheck = state;
			Enabled = true;
		}

		public EntityDesignator(AssetLocation pattern, Color color, EnumEntityState? state, EntityDesignatonAction specialAct)
		{
			Pattern = pattern;
			Color = color;
			StateCheck = state;
			SpecialAction = specialAct;
			SpecialActionName = specialAct.Method.Name;
			Enabled = true;
		}

		public override string ToString()
			=> Pattern.ToShortString() + "|" + Color.Name + "|" + StateCheck ?? "";

		[OnDeserialized]
		public void RelinkDesignator(StreamingContext sCtx)
		{
		//TODO: properly Via reflection - and support for external designators?
		if (SpecialAction == null && !String.IsNullOrEmpty(SpecialActionName)) {
		switch (SpecialActionName) {

		case "KeepTrackOfMerchant":
			SpecialAction = DefaultDesignators.KeepTrackOfMerchant;
			break;

		
		}
		}

		}
	}
}

